from fastapi import APIRouter, Depends, HTTPException, status, Response
from fastapi.responses import JSONResponse
from datetime import datetime
from ..dependencies import users_token, permissions_checker, database
from ..models import ipaddress, users
from pydantic import EmailStr
from typing import Annotated
from bson import ObjectId
router = APIRouter()



@router.get("/ipaddress", tags=["ipaddress"], response_model=list[ipaddress.IpAddressOut])
async def read_events(authorize: Annotated[bool, Depends(permissions_checker.PermissionChecker(roles=["Admin", "User"]))], skip: int = 0, limit: int = 20, id_ip: str | None = None, ip: str | None = None, status: int | None = None):
    if limit < 1 or skip < 0 or limit < skip:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="skip should be greater than 0 and limit should be greater than 1. Limit should be greater than skip"
        )
    limit = limit + skip
    listIp = []
    ip_repository = ipaddress.IpAddressRepository(database=database.database)
    object_search = {}
    if status is not None:
        object_search = {"status":{"$eq": status}}
    if id_ip is not None:
        ipid = ObjectId(id_ip)
        object_search = {"id": {"$regex": ipid}}
        if status is not None:
           object_search = {"$and":[{"id":{"$regex": ipid}}, {"status":{"$eq":status}}]}
    if ip is not None:
        object_search = {"ip": {"$regex": ip}}
        if status is not None:
           object_search = {"$and":[{"ip":{"$regex": ip}}, {"status":{"$eq":status}}]}


    for ip_index in ip_repository.find_by(object_search, limit=limit, skip=skip):
        ip = ipaddress.IpAddressOut(id=ip_index.id, username=ip_index.username, status=ip_index.status, ip=ip_index.ip, socialnetwork=ip_index.socialnetwork)
        listIp.append(ip)
    return listIp


@router.get("/ipaddress/me",tags=["ipaddress"])
async def read_users_me(current_user: Annotated[users.User, Depends(users_token.get_current_active_user)], authorize: Annotated[bool, Depends(permissions_checker.PermissionChecker(roles=["Admin", "User"]))]):
    ip_repository = ipaddress.IpAddressRepository(database=database.database)
    listIp = []

    for ip_index in ip_repository.find_by({"username":{"$eq": current_user.username}}, limit=limit, skip=skip):
        ip = ipaddress.IpAddressOut(id=ip_index.id, username=ip_index.username, status=ip_index.status, ip=ip_index.ip, socialnetwork=ip_index.socialnetwork)
        listIp.append(ip)
    return listIp

@router.get("/ipaddress/count", tags=["ipaddress"])
async def read_events_count(authorize: Annotated[bool, Depends(permissions_checker.PermissionChecker(roles=["Admin", "User"]))]):
    count = database.database.get_collection("ipadress").estimated_document_count()
    content = {"count":count}
    response = JSONResponse(content=content)
    return response


@router.get("/ipaddress/{item_id}", tags=["ipaddress"], response_model=ipaddress.IpAddress)
async def read_events_id(item_id : str, authorize: Annotated[bool, Depends(permissions_checker.PermissionChecker(roles=["Admin"]))]):
    ip_repository = ipaddressIpAddressRepository(database=database.database)
    ip = ip_repository.find_one_by_id(ObjectId(item_id))
    return ip


@router.delete("/ipaddress/me/{item_id}", tags=["ipaddress"])
async def delete_event_me(item_id: str, current_user: Annotated[users.User, Depends(users_token.get_current_active_user)], authorize: Annotated[bool, Depends(permissions_checker.PermissionChecker(roles=["Admin", "User"]))], remove: bool = False):
    ip_repository = ipaddress.IpAddressRepository(database=database.database)
    ip = ip_repository.find_one_by_id(ObjectId(item_id))
    if remove is True:
        ip.deleted_at = datetime.today()
        ip.status = -1
        content = {"message": "ip is deleted"}
    else:
        ip.status = 0
        ip.deleted_at = datetime.today()
        content = {"message": "ip is disabled"}
    ip_repository.save(ip)
    response = JSONResponse(content=content)
    return response

@router.delete("/ipaddress/groups",tags=["ipaddress"])
async def delete_events_groups(authorize: Annotated[bool, Depends(permissions_checker.PermissionChecker(roles=["Admin"]))], remove: bool = False, ipaddressids: ipaddress.IpAddressIDS | None = None):
    if len(ipaddressids.ids) == 0:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="eventids should be greater than 0"
        )
    
    ip_repository = ipaddress.IpAddressRepository(database=database.database)
    for i in eventids.ids:
        event = event_repository.find_one_by_id(ObjectId(i))
        if remove is True:
            event.status = -1
            event.deleted_at = datetime.today()
            content = {"message": "events are deleted "}
        else:
            event.status = 0
            event.disabled_at = datetime.today()
            content = {"message": "events are disabled"}
        event_repository.save(event)
    
    response = JSONResponse(content=content)
    return response

@router.delete("/events/{item_id}", tags=["events"])
async def delete_events_id(item_id : str, authorize: Annotated[bool, Depends(permissions_checker.PermissionChecker(roles=["Admin"]))], remove : bool = False):
    event_repository = events.EventRepository(database=database.database)
    event = event_repository.find_one_by_id(ObjectId(item_id))
    if remove is True:
        event.status = -1
        event.deleted_at = datetime.today()
        content = {"message": "events are deleted"}
    else:
        event.status = 0
        event.disabled_at = datetime.today()
        content = {"message": "events are disabled"}
    event_repository.save(event)
    response = JSONResponse(content=content)  
    return response

@router.put("/events/me/{item_id}",tags=["events"])
async def update_events_me(item_id: str, current_user: Annotated[users.User, Depends(users_token.get_current_active_user)], authorize: Annotated[bool, Depends(permissions_checker.PermissionChecker(roles=["Admin", "User"]))], eventSingle: events.EventIn | None = None):
    event_repository = event.EventRepository(database=database.database)
    event = event_repository.find_one_by_id(ObjectId(item_id))
    event.name = EventSingle.name
    event.place = EventSingle.place
    event.start_date = eventSingle.start_date
    event.end_date = eventSingle.end_date
    event.latitude = eventSingle.latitude
    event.longitude = eventSingle.longitude
    event.updated_at = datetime.today()
    event_repository.save(event)
    content = {"message": "event is updated"}
    response = JSONResponse(content=content)  
    return response

@router.put("/events", tags=["events"], status_code=status.HTTP_201_CREATED)
async def update_events(authorize: Annotated[bool, Depends(permissions_checker.PermissionChecker(roles=["Admin"]))], eventSingle: events.EventIn | None = None):
    if eventSingle is None:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="Body request is empty"
        )
    event_repository = events.EventRepository(database=database.database)

    event = event_repository.find_one_by({"name": {'$eq': eventSingle.name}})
    if event is not None:
        raise HTTPException(
            status_code=status.HTTP_204_NO_CONTENT,
            detail="name"
        )
        

    event = events.Event(name=eventSingle.name,  place=eventSingle.place)
    event.start_date = eventSingle.start_date
    event.end_date = eventSingle.end_date
    event.organizers = eventSingle.organizers
    event.latitude = eventSingle.latitude
    event.longitude = eventSingle.longitude
    event.created_at = datetime.today()
    event_repository.save(event)
    content = {"message": "event is created"}
    response = JSONResponse(content=content, status_code=status.HTTP_201_CREATED)  
    return response


@router.put("/events/{item_id}", tags=["events"], status_code=status.HTTP_200_OK)
async def update_events_id(item_id: str, authorize: Annotated[bool, Depends(permissions_checker.PermissionChecker(roles=["Admin"]))], eventSingle: events.EventIn | None = None, response: Response = Response):
    if eventSingle is None:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="Body request is empty"
        )
    event_repository = events.EventRepository(database=database.database)

    event = event_repository.find_one_by({"id": {'$eq': ObjectId(item_id)}})
    if event is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Event not found"
        )
        
    event.name = eventSingle.name
    event.place = eventSingle.place
    event.start_date = eventSingle.start_date
    event.end_date = eventSingle.end_date
    event.organizers = eventSingle.organizers
    event.latitude = eventSingle.latitude
    event.longitude = eventSingle.longitude
    event.updated_at = datetime.today()
    event_repository.save(event)
    content = {"message": "event is updated"}
    response = JSONResponse(content=content)  
    return response


@router.patch("/events/groups",tags=["events"])
async def patch_events_groups(authorize: Annotated[bool, Depends(permissions_checker.PermissionChecker(roles=["Admin"]))], eventids: events.EventIDS | None = None):
    if len(eventids.ids) == 0:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="eventids should be greater than 0"
        )
    
    event_repository = events.EventRepository(database=database.database)
    content = {"message": "events are enabled"}
    for i in eventids.ids:
        event = event_repository.find_one_by_id(ObjectId(i))
        event.status = 1
        event.disabled_at = None
        event.deleted_at = None
        event_repository.save(event)
    
    
    response = JSONResponse(content=content)
    return response

@router.patch("/events/{item_id}", tags=["events"])
async def patch_events_id(item_id : str, authorize: Annotated[bool, Depends(permissions_checker.PermissionChecker(roles=["Admin"]))]):
    event_repository = events.EventRepository(database=database.database)
    event = event_repository.find_one_by_id(ObjectId(item_id))
    event.status = 1
    event.disabled_at = None
    event.deleted_at = None
    event_repository.save(event)
    content = {"message": "event is enabled"}
    response = JSONResponse(content=content)  
    return response